@@ -377,7 +377,6 @@ dbCellReadDef(f, cellDef, name, ignoreTech)
 	CIFTechInputScale(1, d, TRUE);
 	CIFTechOutputScale(1, d);
 	DRCTechScale(1, d);
-	PlowAfterTech();
 	ExtTechScale(1, d);
 	WireTechScale(1, d);
 #ifdef LEF_MODULE
@@ -1026,6 +1025,23 @@ dbReadOpen(cellDef, name, setFileName, errptr)
 	f = PaLockOpen(cellDef->cd_file, "r", DBSuffix, ".",
 			(char *) NULL, &filename, &is_locked);
 
+	/* Fall back on the original method of using search paths. */
+
+	if (f == NULL)
+	{
+	    f = PaLockOpen(cellDef->cd_name, "r", DBSuffix, Path,
+			CellLibPath, &filename, &is_locked);
+
+	    if (f != NULL)
+	    {
+		if (pptr != NULL) *pptr = '.';
+		TxError("Warning:  Parent cell lists instance of \"%s\" at bad file "
+			"path %s.\n", cellDef->cd_name, cellDef->cd_file);
+		TxError("The cell exists in the search paths at %s.\n", filename);
+		TxError("The discovered version will be used.\n");
+	    }
+	}
+
 	if (errptr != NULL) *errptr = errno;
 	if (pptr != NULL) *pptr = '.';	// Put it back where you found it!
     }
@@ -1156,13 +1172,14 @@ dbReadUse(cellDef, line, len, f, scalen, scaled)
     int scaled;		/* Divide values in file by this */
 {
     int xlo, xhi, ylo, yhi, xsep, ysep, childStamp;
-    int absa, absb, absd, abse;
-    char cellname[1024], useid[1024];
+    int absa, absb, absd, abse, nconv;
+    char cellname[1024], useid[1024], path[1024];
     CellUse *subCellUse;
     CellDef *subCellDef;
     Transform t;
     Rect r;
     bool locked;
+    char *slashptr, *pathptr;
 
     if (strncmp(line, "use", 3) != 0)
     {
@@ -1171,11 +1188,19 @@ dbReadUse(cellDef, line, len, f, scalen, scaled)
     }
 
     useid[0] = '\0';
-    if (sscanf(line, "use %1023s %1023s", cellname, useid) < 1)
+    nconv = sscanf(line, "use %1023s %1023s %1023s", cellname, useid, path);
+    if (nconv < 1)
     {
 	TxError("Malformed \"use\" line: %s", line);
 	return (FALSE);
     }
+    /* Make sure useid[0] is an empty string if no useid was provided */
+    if (nconv == 1) useid[0] = '\0';
+    if (nconv <= 2) path[0] = '\0';
+
+    pathptr = &path[0];
+    while (*pathptr == ' ' || *pathptr == '\t') pathptr++;
+    if (*pathptr == '\n') *pathptr = '\0';
 
     locked = (useid[0] == CULOCKCHAR) ? TRUE : FALSE;
 
@@ -1318,6 +1343,100 @@ badTransform:
 	goto nextLine;
     }
 
+    /* Relative path handling:  If path does not have a leading "/"	*/
+    /* or "~" and cellDef->cd_file has path components, then the path	*/
+    /* should be interpreted relative to the path of the parent cell.	*/
+
+    if ((*pathptr == '\0') || ((*pathptr != '/') && (*pathptr != '~')))
+	if ((cellDef->cd_file != NULL) &&
+			(slashptr = strrchr(cellDef->cd_file, '/')) != NULL)
+	{
+	    *slashptr = '\0';
+	    if (*pathptr == '\0')
+		strcpy(path, cellDef->cd_file);
+	    else
+	    {
+		char savepath[1024];
+		strcpy(savepath, pathptr);
+		sprintf(path, "%s/%s", cellDef->cd_file, savepath);
+	    }
+	    pathptr = &path[0];
+	    *slashptr = '/';
+	}
+
+    /* If path has a leading '~/' and cellDef->cd_file has an absolute	*/
+    /* path that does not match the user's home directory, but appears	*/
+    /* to be a different home directory, then replace the "~" with the	*/
+    /* home directory used by the parent cell.				*/
+
+    if (*pathptr == '~' && *(pathptr + 1) == '/')
+	if ((cellDef->cd_file != NULL) && (cellDef->cd_file[0] == '/'))
+	{
+	    char *homedir = getenv("HOME");
+	    if (strncmp(cellDef->cd_file, homedir, strlen(homedir)) ||
+			*(cellDef->cd_file + strlen(homedir)) != '/')
+	    {
+		char *homeroot = strrchr(homedir, '/');
+		int rootlen = (int)(homeroot - homedir) + 1;
+		if (!strncmp(cellDef->cd_file, homedir, rootlen))
+		{
+		    char savepath[1024];
+		    char *userbrk = strchr(cellDef->cd_file + rootlen, '/');
+		    if (userbrk != NULL)
+		    {
+			int userlen = (int)(userbrk - cellDef->cd_file);
+			strcpy(savepath, pathptr + 1);
+			strcpy(path, cellDef->cd_file);
+			strcpy(path + userlen, savepath);
+			pathptr = &path[0];
+		    }
+		}
+	    }
+	}
+
+    /* If "use" line contains a path name, then set cd_file to this and	*/
+    /* it will be the preferred path.  If cd_file is already set and	*/
+    /* points to a different target, then flag an error, as there are	*/
+    /* now two versions of the same cell name coming from different	*/
+    /* sources, and this must be corrected.				*/
+
+    if (*pathptr != '\0')
+    {
+	if (subCellDef->cd_file != NULL)
+	{
+	    slashptr = strrchr(subCellDef->cd_file, '/');
+	    if (slashptr != NULL)
+	    {
+		*slashptr = '\0';
+
+		if (strcmp(subCellDef->cd_file, pathptr))
+		{
+		    TxError("Duplicate cell in %s:  Instance of cell %s is from "
+				"path %s but cell was previously read from %s.\n",
+				cellDef->cd_name, slashptr + 1, pathptr,
+				subCellDef->cd_file);
+
+		    /* To do:  Check if new path does not exist (ignore),	*/
+		    /* or if new path has same symbolic link or is the same	*/
+		    /* filesize and checksum (ignore).  If file appears to	*/
+		    /* be truly different, then create a new cell with a	*/
+		    /* modified cell name.					*/
+
+		    TxError("New path will be ignored.  Please check.\n");
+		}
+		*slashptr = '/';
+	    }
+	}
+	else
+	{
+	    /* Reconstruct file from path and cellname */
+
+	    strcat(path, "/");
+	    strcat(path, subCellDef->cd_name);
+	    strcat(path, DBSuffix);
+	    StrDup(&subCellDef->cd_file, path);
+	}
+    }
     
     subCellUse = DBCellNewUse(subCellDef, (useid[0]) ?
 		((locked) ? useid + 1 : useid) : (char *) NULL);
@@ -1473,6 +1592,7 @@ dbReadProperties(cellDef, line, len, f, scalen, scaled)
 			cellDef->cd_bbox.r_xtop /= scaled;
 			cellDef->cd_bbox.r_ytop /= scaled;
 		    }
+		    cellDef->cd_extended = cellDef->cd_bbox;
 		    cellDef->cd_flags |= CDFIXEDBBOX;
 		}
 	    }
@@ -2243,6 +2363,7 @@ DBCellWriteFile(cellDef, f)
      * plane being searched.
      */
 
+    arg.wa_name = cellDef->cd_file;
     arg.wa_file = f;
     arg.wa_reducer = reducer;
     for (type = TT_PAINTBASE; type < DBNumUserLayers; type++)
@@ -2454,10 +2575,12 @@ dbWritePropFunc(key, value, cdata)
     ClientData cdata;
 {
     FILE *f = (FILE *)cdata;
-    char lstring[256];
+    char *lstring;
 
+    lstring = (char *)mallocMagic(10 + strlen((char *)value) + strlen(key));
     sprintf(lstring, "string %s %s\n", key, (char *)value);
     FPRINTR(f, lstring);
+    freeMagic(lstring);
 
     return 0;
 }
@@ -2902,20 +3025,44 @@ dbWriteCellFunc(cellUse, cdarg)
     struct writeArg *arg = (struct writeArg *) cdarg;
     Transform *t;
     Rect *b;
-    char     cstring[256], *pathend;
+    char     cstring[256], *pathend, *pathstart, *parent;
 
     t = &(cellUse->cu_transform);
     b = &(cellUse->cu_def->cd_bbox);
+    pathstart = cellUse->cu_def->cd_file;
+    parent = arg->wa_name;
 
-    if (cellUse->cu_def->cd_file == NULL)
+    if (pathstart == NULL)
 	pathend = NULL;
     else
     {
-	pathend = strrchr(cellUse->cu_def->cd_file, '/');
-	if (pathend != NULL) *pathend = '\0';
+	char *slashptr, *pathorigin;
+
+	/* Get child path relative to the parent path */
+
+	pathorigin = pathstart;
+	pathend = strrchr(pathstart, '/');
+	slashptr = strchr(pathstart, '/');
+	while (slashptr)
+	{
+	    if (!strncmp(pathorigin, parent, (int)(slashptr - pathorigin + 1)))
+	    {
+		pathstart = slashptr + 1;
+		slashptr = strchr(pathstart, '/');
+	    }
+	    else
+		break;
+	}
+	if (pathend != NULL)
+	{
+	    *pathend = '\0';
+	    if (pathstart >= pathend)
+		pathstart = NULL;
+	}
     }
 
-    if ((cellUse->cu_def->cd_flags & CDVISITED) || (pathend == NULL))
+    if ((cellUse->cu_def->cd_flags & CDVISITED) || (pathend == NULL) ||
+		(pathstart == NULL) || (*pathstart == '\0'))
     {
 	sprintf(cstring, "use %s %c%s\n", cellUse->cu_def->cd_name,
 		(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
@@ -2923,9 +3070,26 @@ dbWriteCellFunc(cellUse, cdarg)
     }
     else
     {
-	sprintf(cstring, "use %s %c%s %s\n", cellUse->cu_def->cd_name,
-		(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
-		cellUse->cu_id, cellUse->cu_def->cd_file);
+	/* If path starts with home path, then replace with "~"	*/
+	/* to make IP semi-portable between home directories	*/
+	/* with the same file structure.			*/
+
+	char *homedir = getenv("HOME");
+
+	if (!strncmp(cellUse->cu_def->cd_file, homedir, strlen(homedir))
+		&& (*(cellUse->cu_def->cd_file + strlen(homedir)) == '/'))
+	{
+	    sprintf(cstring, "use %s %c%s ~%s\n", cellUse->cu_def->cd_name,
+			(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
+			cellUse->cu_id, cellUse->cu_def->cd_file +
+			strlen(homedir));
+	}
+	else
+	{
+	    sprintf(cstring, "use %s %c%s %s\n", cellUse->cu_def->cd_name,
+			(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
+			cellUse->cu_id, pathstart);
+	}
     }
     FPRINTR(arg->wa_file, cstring);
 
