@@ -191,6 +191,18 @@ ToolGetBox(rootDef, rootArea)
     return TRUE;
 }
 
+/* ToolScaleBox ---  Simple scaling of the root area box, no update needed */
+
+void
+ToolScaleBox(scalen, scaled)
+    int scalen;
+    int scaled;
+{
+    DBScalePoint(&boxRootArea.r_ll, scalen, scaled);
+    DBScalePoint(&boxRootArea.r_ur, scalen, scaled);
+
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -457,6 +469,9 @@ DBWDrawCrosshair(window, plane)
 {
     Point p;
 
+    /* Crosshair is disabled by setting to MINFINITY */
+    if (crosshairPos.p_x == MINFINITY) return;
+
     WindPointToScreen(window, &crosshairPos, &p);
 
     GrSetStuff(STYLE_YELLOW1);
@@ -471,6 +486,16 @@ DBWDrawCrosshair(window, plane)
 		   window->w_screenArea.r_xtop, p.p_y);
 }
 
+/* DBWScaleCrosshair ---  Simple scaling of the crosshair point, no update needed */
+
+void
+DBWScaleCrosshair(scalen, scaled)
+    int scalen;
+    int scaled;
+{
+    DBScalePoint(&crosshairPos, scalen, scaled);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *	DBWSetCrosshair --
