@@ -281,7 +281,7 @@ areaCheck(tile, arg)
  * Side effects:
  *	Calls function for each design-rule violation in celldef
  *	that is triggered by an edge in rect and whose violation
- *	area falls withing clipRect.  This routine makes a flat check:
+ *	area falls within clipRect.  This routine makes a flat check:
  *	it considers only information in the paint planes of celldef,
  *	and does not expand children.  Function should have the form:
  *	void
@@ -778,6 +778,7 @@ drcTile (tile, arg)
 		    arg->dCD_cptr = cptr;
 		    arg->dCD_entries = 0;
 		    TTMaskCom2(&tmpMask, &cptr->drcc_mask);
+		    TTMaskClearType(&tmpMask, TT_ERROR_S);
 		    DBSrPaintArea((Tile *) NULL,
 				arg->dCD_celldef->cd_planes[cptr->drcc_plane],
 				&errRect, &tmpMask, areaCheck, (ClientData) arg);
@@ -1082,6 +1083,7 @@ checkbottom:
 		    arg->dCD_cptr = cptr;
 		    arg->dCD_entries = 0;
 		    TTMaskCom2(&tmpMask, &cptr->drcc_mask);
+		    TTMaskClearType(&tmpMask, TT_ERROR_S);
 		    DBSrPaintArea((Tile *) NULL,
 				arg->dCD_celldef->cd_planes[cptr->drcc_plane],
 				&errRect, &tmpMask, areaCheck, (ClientData) arg);
